<?php

namespace Termageddon\MainWP;

class Termageddon_For_Mainwp_Main {

	public static $instance = null;
	public $ver             = '2.0';

	public static function get_instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	public function __construct() {

		add_action( 'init', array( &$this, 'localization' ) );
		add_action( 'admin_init', array( &$this, 'admin_init' ) );
		add_filter( 'mainwp_main_menu_submenu', array( $this, 'init_menu_submenu' ), 10, 1 );
		add_filter( 'mainwp_security_nonces', array( &$this, 'hook_security_nonces' ), 10, 2 );

		add_action( 'mainwp_before_bulkpost_editor', array( &$this, 'before_bulkpost_editor' ), 10, 1 );
		add_action( 'mainwp_save_bulkpage', array( &$this, 'save_meta_bulkpage' ), 9, 1 ); // default bulkpost acction order is 10.
		add_action( 'mainwp_bulkpost_edit_top_side', array( &$this, 'bulkpost_edit_top_side' ), 10, 2 );
		add_filter( 'mainwp_bulkpost_edit_title', array( &$this, 'bulkpost_edit_title' ), 10, 2 );
		add_action( 'mainwp_top_bulkpost_edit_content', array( &$this, 'top_bulkpost_edit_content' ), 10, 2 );
		add_filter( 'mainwp_bulkpost_select_sites_settings', array( &$this, 'bulkpost_select_sites_settings' ), 10, 2 );
		add_filter( 'mainwp_bulkpost_editor_settings', array( &$this, 'bulkpost_editor_settings' ), 10, 2 );

		add_filter( 'mainwp_pre_posting_posts', array( &$this, 'pre_posting_page' ), 8, 2 );
	}

	public function localization() {
		load_plugin_textdomain( 'termageddon-for-mainwp', false, dirname( dirname( plugin_basename( __FILE__ ) ) ) . '/languages/' );
	}

	public function admin_init() {

		if ( isset( $_GET['page'] ) && ( 'Extensions-Termageddon-For-Mainwp' == $_GET['page'] ) ) {
			wp_enqueue_script( 'termageddon-for-mainwp', TERMAGEDDON_FOR_MAINWP_URL . '/js/termageddon-extension.js', array( 'jquery' ), $this->ver, true );
		}

		Termageddon_For_Mainwp_Page::get_instance()->admin_init();

		add_action( 'mainwp_update_site', array( &$this, 'mainwp_update_site' ), 8, 1 );
		add_action( 'mainwp_delete_site', array( &$this, 'mainwp_delete_site' ), 8, 1 );
		add_filter( 'mainwp_page_navigation', array( $this, 'init_page_navigation' ), 10, 2 );
	}

	public function init_menu_submenu( $subs ) {
		if ( is_array( $subs ) && isset( $subs['PageBulkManage'] ) ) {
			$subs['PageBulkManage'][] = array( __( 'New Termageddon' ), 'admin.php?page=PageBulkAdd&termageddon=yes', false );
		}
		return $subs;
	}

	public function before_bulkpost_editor( $post = false ) {
		if ( isset( $_GET['termageddon'] ) && 'yes' == $_GET['termageddon'] ) {
			echo '<h3>' . esc_html__( 'Termageddon Embed code', 'termageddon-for-mainwp' ) . '</h3>';
		}
	}

	public function bulkpost_editor_settings( $input, $post = false ) {
		if ( isset( $_GET['termageddon'] ) ) {
			if ( ! is_array( $input ) ) {
				$input = array();
			}
			$input = array_merge(
				$input,
				array(
					'quicktags'     => false,
					'media_buttons' => false,
					'tinymce'       => false,
				)
			);

		}
		return $input;
	}

	public function bulkpost_edit_title( $title, $post = false ) {

		if ( isset( $_GET['termageddon'] ) && $post && 'bulkpage' == $post->post_type ) {
			$type_term = $this->get_type_of_term( $post );
			if ( empty( $type_term ) ) {
				return $title;
			}
			$new   = ( 'auto-draft' === $post->post_status ) ? true : false;
			$title = $this->get_termageddon_title( $type_term, $new );
		}

		return $title;
	}

	public function top_bulkpost_edit_content( $post = false ) {
		if ( isset( $_GET['page'] ) && 'PageBulkEdit' == $_GET['page'] && isset( $_GET['termageddon'] ) && $post && 'bulkpage' == $post->post_type ) {
			$sites_posts = get_post_meta( $post->ID, '_termageddon_4_mainwp_sites_posts', true );
			$site_id     = get_post_meta( $post->ID, '_mainwp_edit_post_site_id', true );
			if ( is_array( $sites_posts ) && $site_id ) {
				$link_to = '';
				if ( isset( $sites_posts['updated_posts'] ) && ! empty( $sites_posts['updated_posts'] ) ) {
					if ( isset( $sites_posts['updated_posts'][ $site_id ] ) && isset( $sites_posts['updated_posts'][ $site_id ]['link'] ) ) {
						$link_to = $sites_posts['updated_posts'][ $site_id ]['link'];
					}
				} elseif ( isset( $sites_posts['previous_posts'] ) ) {
					if ( isset( $sites_posts['previous_posts'][ $site_id ] ) && isset( $sites_posts['previous_posts'][ $site_id ]['link'] ) ) {
						$link_to = $sites_posts['previous_posts'][ $site_id ]['link'];
					}
				}

				if ( ! empty( $link_to ) ) {
					echo '<a href="' . esc_url_raw( $link_to ) . '" target="_blank"><h3 style="margin-top:10px;">' . esc_html( $link_to ) . '</h3></a>';
				}
			}
		}
	}

	public function get_termageddon_title( $type, $new = true ) {
		if ( $new ) {
			$title = esc_html__( 'Create New Policy Page', 'termageddon-for-mainwp' );
			if ( 'terms' == $type ) {
				$title = esc_html__( 'Create New Terms of Service Page', 'termageddon-for-mainwp' );
			} elseif ( 'disclaimer' == $type ) {
				$title = esc_html__( 'Create New Disclaimer Page', 'termageddon-for-mainwp' );
			} elseif ( 'eula' == $type ) {
				$title = esc_html__( 'Create New EULA Page', 'termageddon-for-mainwp' );
			}
		} else {
			$title = esc_html__( 'Edit Policy Page', 'termageddon-for-mainwp' );
			if ( 'terms' == $type ) {
				$title = esc_html__( 'Edit Terms of Service Page', 'termageddon-for-mainwp' );
			} elseif ( 'disclaimer' == $type ) {
				$title = esc_html__( 'Edit Disclaimer Page', 'termageddon-for-mainwp' );
			} elseif ( 'eula' == $type ) {
				$title = esc_html__( 'Edit EULA Page', 'termageddon-for-mainwp' );
			}
		}
		return $title;
	}

	public function save_meta_bulkpage( $post_id ) {

		// verify if this is an auto save routine. If it is our form has not been submitted, so we dont want to do anything.
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return;
		}

		if ( ! isset( $_POST['termageddon_for_mainwp_nonce'] ) || ! wp_verify_nonce( $_POST['termageddon_for_mainwp_nonce'], 'termageddon_' . $post_id ) ) {
			return false;
		}

		add_filter( 'redirect_post_location', array( $this, 'redirect_edit_bulkpost' ), 11, 2 ); // priority 11 to adding parameter.

		Termageddon_For_Mainwp_Page::get_instance()->handle_metabox_posting( $post_id );
	}

	public function redirect_edit_bulkpost( $location, $post_id ) {
		$location .= '&termageddon=yes';
		return $location;
	}

	public function get_type_of_term( $post = false ) {
		if ( isset( $_GET['termageddon'] ) && isset( $_GET['addnew'] ) ) {
			$type_term = isset( $_GET['addnew'] ) ? sanitize_text_field( wp_unslash( $_GET['addnew'] ) ) : '';
		} elseif ( isset( $_GET['termageddon'] ) && ! isset( $_GET['addnew'] ) && isset( $_GET['page'] ) && 'PageBulkAdd' == $_GET['page'] ) {
			$type_term = 'privacy';
		} elseif ( is_object( $post ) ) {
			$type_term = get_post_meta( $post->ID, '_termageddon_4_mainwp_type', true );
		}

		if ( ! in_array( $type_term, array( 'privacy', 'terms', 'disclaimer', 'eula' ) ) ) {
			$type_term = '';
		}

		return $type_term;
	}

	public function hook_security_nonces( $input ) {
		if ( ! is_array( $input ) ) {
			$input = array();
		}
		$actions = array(
			'termageddon_for_mainwp_delete_site_post',
			'termageddon_for_mainwp_delete_post',
		);
		foreach ( $actions as $act ) {
			$input[ $act ] = wp_create_nonce( $act );
		}
		return $input;
	}

	public function bulkpost_edit_top_side( $post, $post_type = false ) {
		if ( ! isset( $_GET['termageddon'] ) || 'yes' != $_GET['termageddon'] ) {
			return;
		}

		if ( 'bulkpage' != $post->post_type ) {
			return;
		}

		$type_term = $this->get_type_of_term( $post );
		if ( empty( $type_term ) ) {
			return;
		}

		$new = ( 'auto-draft' === $post->post_status ) ? true : false;

		?>
		<div id="mainwp-termageddon-types" class="mainwp-search-options">
			<div class="ui header"><?php esc_html_e( 'Termageddon page types', 'termageddon-for-mainwp' ); ?></div>
			<div class="field">
				<div class="ui relaxed list">
					<div class="ui radio checkbox item">
						<input type="radio" name="termageddon_for_mainwp_type" value="privacy" term-title="<?php echo $this->get_termageddon_title( 'privacy', $new ); ?>" <?php echo ( 'privacy' == $type_term ) ? 'checked="checked"' : ''; ?> >
						<label><?php esc_html_e( 'Privacy Policy', 'termageddon-for-mainwp' ); ?></label>
					</div>
					<div class="ui radio checkbox item">
						<input type="radio" name="termageddon_for_mainwp_type" value="terms" term-title="<?php echo $this->get_termageddon_title( 'terms', $new ); ?>" <?php echo ( 'terms' == $type_term ) ? 'checked="checked"' : ''; ?>>
						<label><?php esc_html_e( 'Terms of Service', 'termageddon-for-mainwp' ); ?></label>
					</div>
					<div class="ui radio checkbox item">
						<input type="radio" name="termageddon_for_mainwp_type" value="disclaimer" term-title="<?php echo $this->get_termageddon_title( 'disclaimer', $new ); ?>" <?php echo ( 'disclaimer' == $type_term ) ? 'checked="checked"' : ''; ?>>
						<label><?php esc_html_e( 'Disclaimer', 'termageddon-for-mainwp' ); ?></label>
					</div>
					<div class="ui radio checkbox item">
						<input type="radio" name="termageddon_for_mainwp_type" value="eula" term-title="<?php echo $this->get_termageddon_title( 'eula', $new ); ?>" <?php echo ( 'eula' == $type_term ) ? 'checked="checked"' : ''; ?>>
						<label><?php esc_html_e( 'EULA', 'termageddon-for-mainwp' ); ?></label>
					</div>
				</div>
			</div>
		</div>
		<div class="ui divider"></div>
		<input type="hidden" name="termageddon_for_mainwp_nonce" value="<?php echo wp_create_nonce( 'termageddon_' . $post->ID ); ?>">
		<?php

		$disable_siteids = array();

		$types = array( 'privacy', 'terms', 'disclaimer', 'eula' );

		// to disable select sites have the page.
		if ( $new ) {
			$query_args = array(
				'post_type'      => 'bulkpage',
				'posts_per_page' => -1,
				'post_status'    => array( 'publish', 'pending', 'future', 'private', 'draft' ),
				'post_parent'    => null,
				'orderby'        => 'date',
				'order'          => 'DESC',
			);

			$query_args['meta_query'] = array(
				array(
					'key'     => '_termageddon_4_mainwp',
					'value'   => 'yes',
					'compare' => '=',
				),
				array(
					'key'     => '_termageddon_4_mainwp_type',
					'value'   => $types,
					'compare' => 'IN',
				),
			);

			$query = new \WP_Query( $query_args );

			if ( $query->found_posts ) {
				foreach ( $query->posts as $post ) {

					$siteid = get_post_meta( $post->ID, '_selected_sites', true );
					$siteid = is_array( $siteid ) ? current( $siteid ) : $siteid;
					$type   = get_post_meta( $post->ID, '_termageddon_4_mainwp_type', true );

					$disable_siteids[ $type ][] = $siteid;
				}
			}
		}

		$dis_edit_siteids = isset( $disable_siteids[ $type_term ] ) ? implode( ',', $disable_siteids[ $type_term ] ) : '';
		?>
		<script type='text/javascript'>
			jQuery(document).ready(function($){
				var disable_checks = {};
				<?php
				foreach ( $types as $type ) {
					?>
					disable_checks['<?php echo esc_js( $type ); ?>'] = [<?php echo isset( $disable_siteids[ $type ] ) ? implode( ',', $disable_siteids[ $type ] ) : ''; ?>]
					<?php
				}
				?>
				$('#mainwp-termageddon-types input[type=radio]').change( function () {						
					$('#bulkpost-title').text( $(this).attr('term-title') );
					$('#mainwp-select-sites-list input[type=radio]').removeAttr('disabled');
					$.each( disable_checks[$(this).val()], function(index,value){
						$('#mainwp-select-sites-list input[type=radio][siteid=' + value + ']').attr('disabled', 'disabled');
					});
				});
				$.each( [<?php echo esc_js( $dis_edit_siteids ); ?>], function( index, value ){
					$('#mainwp-select-sites-list input[type=radio][siteid=' + value + ']').attr('disabled', 'disabled');
				});
			});
		</script>
		<?php
	}

	public function bulkpost_select_sites_settings( $settings, $post = false ) {
		if ( isset( $_GET['termageddon'] ) && 'yes' == $_GET['termageddon'] ) {
			return array(
				'type'            => 'radio',
				'show_group'      => false,
				'show_select_all' => false,
			);
		}
	}

	public function init_page_navigation( $sub_headers, $caller_class = false ) {

		$caller_class = str_replace( 'MainWP\\Dashboard\\', '', $caller_class ); // remove namespace string.

		$new_item = array(
			'href'   => 'admin.php?page=PageBulkAdd&termageddon=yes',
			'title'  => __( 'New Termageddon', 'termageddon-for-mainwp' ),
			'active' => false,
		);

		$item_id = isset( $_GET['post_id'] ) ? intval( $_GET['post_id'] ) : '';

		$edit_item = array(
			'href'   => 'admin.php?page=PageBulkEdit&termageddon=yes&post_id=' . $item_id,
			'title'  => __( 'Edit Termageddon', 'termageddon-for-mainwp' ),
			'active' => true,
		);

		$add_new_items      = false;
		$add_edit_items     = false;
		$clear_other_active = false;

		if ( 'MainWP_Page' == $caller_class ) { // page navigation of bulkpage.
			$add_new_items = true;
			// editing termageddon.
			if ( isset( $_GET['page'] ) && 'PageBulkEdit' == $_GET['page'] && isset( $_GET['termageddon'] ) && 'yes' == $_GET['termageddon'] ) {
				$add_edit_items     = true;
				$clear_other_active = true;
			}
		}

		if ( $add_new_items ) {
			if ( isset( $_GET['page'] ) && 'PageBulkAdd' == $_GET['page'] && isset( $_GET['termageddon'] ) && 'yes' == $_GET['termageddon'] ) {
				$new_item['active'] = true;
				$clear_other_active = true;
			}
		}

		// to un-active other header items first.
		if ( $clear_other_active ) {
			$count = count( $sub_headers );
			for ( $i = 0; $i < $count; $i++ ) {
				if ( isset( $sub_headers[ $i ] ) && isset( $sub_headers[ $i ]['active'] ) ) {
					unset( $sub_headers[ $i ]['active'] );
				}
			}
		}

		if ( $add_new_items ) {
			$sub_headers[] = $new_item; // add Edit header item.
		}

		if ( $add_edit_items ) {
			$sub_headers[] = $edit_item; // add Edit header item.
		}

		return $sub_headers;
	}

	public function pre_posting_page( $params, $website ) {

		$new_post = unserialize( base64_decode( $params['new_post'] ) ); // phpcs:ignore -- compatible.

		if ( isset( $new_post['mainwp_post_id'] ) && $new_post['mainwp_post_id'] ) {
			$post_id = $new_post['mainwp_post_id'];
			if ( 'yes' == get_post_meta( $post_id, '_termageddon_4_mainwp', true ) ) {
				// get child site post ID for this dashboard post.
				$sites_posts = get_post_meta( $post_id, '_termageddon_4_mainwp_sites_posts', true );
				if ( is_array( $sites_posts ) && isset( $sites_posts['previous_posts'] ) ) {
					$previous_posts = $sites_posts['previous_posts'];
					if ( is_array( $previous_posts ) ) {
						if ( isset( $previous_posts[ $website->id ] ) && isset( $previous_posts[ $website->id ]['post_id'] ) && $previous_posts[ $website->id ]['post_id'] ) {
							// set ID to update post on child site.
							$new_post['ID'] = $previous_posts[ $website->id ]['post_id'];
						}
					}
				}

				$params['new_post'] = base64_encode( serialize( $new_post ) ); // phpcs:ignore -- compatible.
			}
		}
		return $params;
	}


	public function mainwp_delete_site( $website ) {

		if ( $website ) {
			$query_args = array(
				'post_type'      => 'bulkpage',
				'posts_per_page' => -1,
				'post_status'    => array( 'publish', 'pending', 'future', 'private', 'draft' ),
				'post_parent'    => null,
			);

			$query_args['meta_query'] = array(
				array(
					'key'     => '_termageddon_4_mainwp',
					'value'   => 'yes',
					'compare' => '=',
				),
				array(
					'key'     => '_mainwp_edit_post_site_id',
					'value'   => $website->id,
					'compare' => '=',
				),
			);

			$query = new \WP_Query( $query_args );
			if ( $query->found_posts ) {
				foreach ( $query->posts as $post ) {
					if ( 'bulkpage' == $post->post_type ) {
						wp_delete_post( $post->ID );
					}
				}
			}
		}
	}
}
