<?php
namespace Termageddon\MainWP;

class Termageddon_For_Mainwp_Page {

	private static $instance = null;

	public static function get_instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	public function admin_init() {
		add_action( 'wp_ajax_termageddon_for_mainwp_delete_site_post', array( &$this, 'ajax_delete_site_post' ) );
		add_action( 'wp_ajax_termageddon_for_mainwp_delete_post', array( &$this, 'ajax_list_delete_post' ) );
		add_filter( 'mainwp-after-posting-bulkpage-result', array( &$this, 'after_posting_bulkpage' ), 10, 4 );
	}

	public function handle_metabox_posting( $post_id ) {
		$post = get_post( $post_id );
		if ( 'bulkpage' == $post->post_type ) {

			update_post_meta( $post_id, '_termageddon_4_mainwp', 'yes' );
			update_post_meta( $post_id, '_termageddon_4_mainwp_type', ( isset( $_POST['termageddon_for_mainwp_type'] ) ? strtolower( sanitize_text_field( wp_unslash( $_POST['termageddon_for_mainwp_type'] ) ) ) : '' ) );

			$siteid = get_post_meta( $post_id, '_selected_sites', true );
			$siteid = is_array( $siteid ) ? current( $siteid ) : $siteid;
			update_post_meta( $post_id, '_mainwp_edit_post_site_id', $siteid ); // to supported selected one site only.
		}
	}

	public function ajax_delete_site_post() {
		do_action( 'mainwp_secure_request', 'termageddon_for_mainwp_delete_site_post' );
		global $termageddonForMainWPActivator;
		$ret               = array( 'status' => false );
		$site_id           = intval( $_POST['site_id'] );
		$post_id           = intval( $_POST['post_id'] );
		$dashboard_post_id = intval( $_POST['dashboard_post_id'] );

		if ( $site_id && $post_id ) {
			$post_data = array(
				'id'     => $post_id,
				'action' => 'delete',
			);
			$ret       = apply_filters( 'mainwp_fetchurlauthed', $termageddonForMainWPActivator->get_child_file(), $termageddonForMainWPActivator->get_child_key(), $site_id, 'post_action', $post_data );
			if ( $dashboard_post_id ) {
				$sites_posts = get_post_meta( $dashboard_post_id, '_termageddon_4_mainwp_sites_posts', true );
				if ( isset( $sites_posts['previous_posts'] ) && isset( $sites_posts['previous_posts'][ $site_id ] ) && isset( $sites_posts['previous_posts'][ $site_id ]['post_id'] ) && $sites_posts['previous_posts'][ $site_id ]['post_id'] == $post_id ) {
					unset( $sites_posts['previous_posts'][ $site_id ] );
				}
				update_post_meta( $dashboard_post_id, '_termageddon_4_mainwp_sites_posts', $sites_posts );
			}
		}
		echo wp_json_encode( $ret );
		exit;
	}

	public function ajax_list_delete_post() {
		do_action( 'mainwp_secure_request', 'termageddon_for_mainwp_delete_post' );
		global $wpdb;
		global $termageddonForMainWPActivator;

		$post_id = intval( $_REQUEST['post_id'] );
		$site_id = intval( $_REQUEST['site_id'] );

		$ret = array();

		if ( $post_id ) {
			$child_post_id = false;
			// get child site post ID for this dashboard post.
			$sites_posts = get_post_meta( $post_id, '_termageddon_4_mainwp_sites_posts', true );

			if ( is_array( $sites_posts ) && isset( $sites_posts['previous_posts'] ) ) {
				$previous_posts = $sites_posts['previous_posts'];
				if ( is_array( $previous_posts ) && isset( $previous_posts[ $site_id ] ) ) {
					if ( ! empty( $previous_posts[ $site_id ]['post_id'] ) ) {
						// set ID to update post on child site.
						$child_post_id = $previous_posts[ $site_id ]['post_id'];
					}
				}
			}

			if ( wp_delete_post( $post_id ) ) {
				if ( $child_post_id ) {
					$post_data = array(
						'id'     => $child_post_id,
						'action' => 'delete',
					);
					$ret       = apply_filters( 'mainwp_fetchurlauthed', $termageddonForMainWPActivator->get_child_file(), $termageddonForMainWPActivator->get_child_key(), $site_id, 'post_action', $post_data );
				}
			}
		}

		echo wp_json_encode( $ret );
		exit;
	}

	public function after_posting_bulkpage( $input, $post, $dbwebsites, $output ) {
		if ( $post ) {
			if ( 'yes' == get_post_meta( $post->ID, '_termageddon_4_mainwp', true ) ) {

				update_post_meta( $post->ID, '_bulkpost_do_not_del', 'yes' ); // to prevent dashboard delete this page.

				$this->render_after_posting( $post, $dbwebsites, $output );

				return true; // return true to output.
			}
		}

		return $input; // return false to no output.
	}

	public function render_after_posting( $post, $dbwebsites, $output ) { // phpcs:ignore -- ignored.

		// calculate what post is create new, updated or will delete.
		$sites_posts = self::bulkposting_posting_done_results( $post, $dbwebsites, $output );
		if ( ! is_array( $sites_posts ) ) {
			$sites_posts = array();
		}
		global $termageddonForMainWPActivator;

		$new_posts     = isset( $sites_posts['new_posts'] ) ? $sites_posts['new_posts'] : array();
		$updated_posts = isset( $sites_posts['updated_posts'] ) ? $sites_posts['updated_posts'] : array();
		$askto_delete  = isset( $sites_posts['ask_to_delete'] ) ? $sites_posts['ask_to_delete'] : array();
		$get_siteids   = isset( $sites_posts['all_site_ids'] ) ? $sites_posts['all_site_ids'] : array();

		$websites = array();
		if ( ! empty( $get_siteids ) ) {
			$websites = apply_filters( 'mainwp_getdbsites', $termageddonForMainWPActivator->get_child_file(), $termageddonForMainWPActivator->get_child_key(), $get_siteids, array() );
		}

		$all_sites = array();
		if ( is_array( $websites ) ) {
			foreach ( $websites as $website ) {
				$all_sites[ $website->id ] = $website;
			}
		}

		?>
		<div class="ui relaxed list">
			<?php
			if ( is_array( $new_posts ) ) {
				foreach ( $new_posts as $site_id => $_post ) {
					if ( isset( $all_sites[ $site_id ] ) ) {
						$website = $all_sites[ $site_id ];
						echo '<div class="item">' . esc_html( $website->name ) . ': New page created. <a href="' . $_post['link'] . '"  target=\"_blank\">View Page</a></div>';
					}
				}
			}

			if ( is_array( $updated_posts ) ) {
				foreach ( $updated_posts as $site_id => $_post ) {
					if ( isset( $all_sites[ $site_id ] ) ) {
						$website = $all_sites[ $site_id ];
						echo '<div class="item">' . esc_html( $website->name ) . ': Page updated. <a href="' . $_post['link'] . '"  target=\"_blank\">View Page</a></div>';
					}
				}
			}
			if ( is_array( $askto_delete ) ) {
				foreach ( $askto_delete as $site_id => $_post ) {
					if ( isset( $all_sites[ $site_id ] ) ) {
						$website = $all_sites[ $site_id ];
						echo '<div class="item termage_bulkpost_posting_item">' . esc_html( $website->name ) . ': <span class="termage_bulkpost_delete_status">Delete the page? <span class="termage_bulkpost_delete_process hidden"></span><a href="#" class="termage_bulkpost_delete_post" termageddon_id="' . $post->ID . '"  site_id="' . $site_id . '" post_id="' . $_post['post_id'] . '">Delete</a> | <a href="' . $_post['link'] . '"  target=\"_blank\">View Page</a></span></div>';
					}
				}
			}
			foreach ( $dbwebsites as $website ) {
				if ( ! isset( $output->ok[ $website->id ] ) ) {
					$err = $output->errors[ $website->id ];

					if ( 'Error - Empty post id' == $err ) {
						$err = ': Not found the post on child site to save.';
					} else {
						$err = ': ERROR - ' . $err;
					}

					echo '<div class="item">' . $website->name . ' ' . $err . '</div>';
				}
			}
			?>
		</div>
		<br/>
		<a href="<?php echo get_admin_url(); ?>admin.php?page=PageBulkAdd&termageddon=yes" class="ui green button" target="_top"><?php esc_html_e( 'Add New Termageddon', 'termageddon-for-mainwp' ); ?></a>
		<a href="<?php echo get_admin_url(); ?>admin.php?page=Extensions-Termageddon-For-Mainwp" class="ui green button" target="_top"><?php esc_html_e( 'Return to Termageddon', 'termageddon-for-mainwp' ); ?></a>

		<script type="text/javascript">
			jQuery(document).ready(function($) {
				jQuery(document).on( 'click', '.termage_bulkpost_delete_post', function(e) {
					if (confirm(__('Are you sure?'))) {
						var parent = $(this).closest('.termage_bulkpost_posting_item');							
						parent.find('.termage_bulkpost_delete_process').html('<i class="notched circle loading icon"></i> Deleting... ').show();
						$(this).hide();
						jQuery.post(ajaxurl, mainwp_secure_data( {
							action: 'termageddon_for_mainwp_delete_site_post',
							post_id: $(this).attr('post_id'),
							site_id: $(this).attr('site_id'),
							dashboard_post_id: $(this).attr('termageddon_id')
						} ), function(data) {
							parent.find('.termage_bulkpost_delete_process').hide();
							if (data && data['status'] == 'SUCCESS') {
								parent.find('.termage_bulkpost_delete_status').html('<i class="green check icon"></i> '+'The page was deleted successfully.');
								} else {
								parent.find('.termage_bulkpost_delete_status').html('<i class="red times icon"></i> Can not delete the page.');
							}
						}, 'json');
					}
					return false;
				});
			});
		</script>
		<?php
	}

	// This function calculate if post is to be updated, deleted or created.
	public static function bulkposting_posting_done_results( $post, $dbwebsites, $output ) { // phpcs:ignore -- ignored.
		if ( ! $post ) {
			return;
		}

		if ( ! $post || 'bulkpage' != $post->post_type ) {
			return;
		}

		if ( get_post_meta( $post->ID, '_termageddon_4_mainwp', true ) != 'yes' ) {
			return;
		}

		$sites_posts = get_post_meta( $post->ID, '_termageddon_4_mainwp_sites_posts', true );

		$just_posts     = array();
		$previous_posts = array();
		$new_posts      = array();
		$updated_posts  = array();
		$askto_delete   = array();
		$all_siteids    = array();

		if ( is_array( $sites_posts ) && isset( $sites_posts['previous_posts'] ) ) {
			$previous_posts = $sites_posts['previous_posts'];
		}

		if ( ! is_array( $previous_posts ) ) {
			$previous_posts = array();
		}

		if ( is_array( $dbwebsites ) ) {
			foreach ( $dbwebsites as $website ) {
				if ( isset( $output->errors[ $website->id ] ) ) {
					$err = $output->errors[ $website->id ];
					if ( __( 'Post ID not found. Please, try again.', 'termageddon-for-mainwp' ) == $err ) {
						if ( isset( $previous_posts[ $website->id ] ) ) {
								unset( $previous_posts[ $website->id ] ); // not found the post on child site to update so remove in the previous list.
						}
					}
				} elseif ( isset( $output->ok[ $website->id ] ) && ( 1 == $output->ok[ $website->id ] ) && ( isset( $output->added_id[ $website->id ] ) ) ) {
					$just_posts[ $website->id ] = array(
						'post_id' => $output->added_id[ $website->id ],
						'link'    => $output->link[ $website->id ],
					);
				}
				if ( ! in_array( $website->id, $all_siteids ) ) {
					$all_siteids[] = $website->id;
				}
			}
		}

		if ( is_array( $previous_posts ) ) {
			foreach ( $previous_posts as $site_id => $_post ) {
				if ( isset( $just_posts[ $site_id ] ) && $just_posts[ $site_id ]['post_id'] == $_post['post_id'] ) {
					$updated_posts[ $site_id ] = $just_posts[ $site_id ];
				} elseif ( ! isset( $just_posts[ $site_id ] ) ) {
					$askto_delete[ $site_id ] = $previous_posts[ $site_id ];
				}
				if ( ! in_array( $site_id, $all_siteids ) ) {
					$all_siteids[] = $site_id;
				}
			}
		}

		if ( ! is_array( $previous_posts ) || count( $previous_posts ) <= 0 ) {
			$new_posts = $just_posts;
		} elseif ( is_array( $just_posts ) && is_array( $previous_posts ) ) {
			foreach ( $just_posts as $site_id => $_post ) {
				if ( ! isset( $previous_posts[ $site_id ] ) || ( isset( $previous_posts[ $site_id ] ) && $previous_posts[ $site_id ]['post_id'] != $_post['post_id'] ) ) {
					$new_posts[ $site_id ] = $_post;
				}
			}
		}

		// to fix create new posts/pages issue.
		foreach ( $just_posts as $_siteid => $value ) {
			if ( ! isset( $previous_posts[ $_siteid ] ) ) {
				$previous_posts[ $_siteid ] = $value; // saved posts/pages was created on child sites.
			}
		}

		$sites_posts                   = array();
		$sites_posts['previous_posts'] = $previous_posts;
		$sites_posts['new_posts']      = $new_posts;
		$sites_posts['updated_posts']  = $updated_posts;
		$sites_posts['ask_to_delete']  = $askto_delete;
		update_post_meta( $post->ID, '_termageddon_4_mainwp_sites_posts', $sites_posts );

		$sites_posts['all_site_ids'] = $all_siteids;
		return $sites_posts;
	}


	public function render_pages_table() {
		?>
		<div class="ui segment">
			<div class="ui message" id="mainwp-message-zone" style="display:none"></div>
		<?php if ( isset( $_REQUEST['message'] ) ) : ?>
			<div class="ui green message"></div>
		<?php endif; ?>
			<div id="termageddon-for-mainwp-pages">
		<?php
		$query_args = array(
			'post_type'      => 'bulkpage',
			'posts_per_page' => -1,
			'post_status'    => array( 'publish', 'pending', 'future', 'private', 'draft' ),
			'post_parent'    => null,
			'orderby'        => 'date',
			'order'          => 'DESC',
		);

		$query_args['meta_query'] = array(
			array(
				'key'     => '_termageddon_4_mainwp',
				'value'   => 'yes',
				'compare' => '=',
			),
			array(
				'key'     => '_termageddon_4_mainwp_type',
				'value'   => array( 'privacy', 'terms', 'disclaimer', 'eula' ),
				'compare' => 'IN',
			),
		);

		$query = new \WP_Query( $query_args );

		$the_pages = array();
		$site_ids  = array();
		if ( $query->found_posts ) {
			foreach ( $query->posts as $post ) {
				$siteid = get_post_meta( $post->ID, '_selected_sites', true );
				$siteid = is_array( $siteid ) ? current( $siteid ) : $siteid;
				$type   = get_post_meta( $post->ID, '_termageddon_4_mainwp_type', true );
				if ( $siteid ) {
					if ( ! isset( $the_pages[ $siteid ] ) || ! isset( $the_pages[ $siteid ][ $type ] ) ) {
						$the_pages[ $siteid ][ $type ] = $post->ID;
					}
				}
			}
		}

		?>
			<table id="termageddon-for-mainwp-pages-table" class="ui stackable table">
				<thead>
					<tr>
						<th><?php esc_html_e( 'Site', 'termageddon-for-mainwp' ); ?></th>
						<th><?php esc_html_e( 'URL', 'termageddon-for-mainwp' ); ?></th>
						<th><?php esc_html_e( 'Privacy Policy', 'termageddon-for-mainwp' ); ?></th>
						<th><?php esc_html_e( 'Terms of Service', 'termageddon-for-mainwp' ); ?></th>
						<th><?php esc_html_e( 'Disclaimer', 'termageddon-for-mainwp' ); ?></th>
						<th><?php esc_html_e( 'EULA', 'termageddon-for-mainwp' ); ?></th>
						<th class="collapsing no-sort"></th>
					</tr>
				</thead>
				<tbody>
			<?php $this->render_table_content( $the_pages ); ?>
				</tbody>
			</table>
		</div>
		</div>
	<script type="text/javascript">
		jQuery( document ).ready( function( $ ) {
			try {
				jQuery( 'table' ).DataTable( {
					"colReorder" : true,
					"stateSave":  true,
					"pagingType": "full_numbers",
					"order": [],
					"language": { "emptyTable": "No Termageddon page created yet." },
					"columnDefs": [ {
					"targets": 'no-sort',
					"orderable": false
					} ],
					"drawCallback": function( settings ) {
						if (typeof mainwp_datatable_fix_menu_overflow === 'function') {
							mainwp_datatable_fix_menu_overflow();
						};
					},
				} );
			} catch ( err ) {
				console.log( err );
				// catch error.
			}
			if (typeof mainwp_datatable_fix_menu_overflow === 'function') {
				mainwp_datatable_fix_menu_overflow();
			};
		} );
		</script>
		<?php
	}

	public function render_table_content( $termageddon_pages ) {

		global $termageddonForMainWPActivator;

		$websites = apply_filters( 'mainwp_getsites', $termageddonForMainWPActivator->get_child_file(), $termageddonForMainWPActivator->get_child_key(), null );

		if ( empty( $websites ) ) {
			return;
		}

		$terms = array( 'privacy', 'terms', 'disclaimer', 'eula' );

		foreach ( $websites as $website ) :
			$site_id    = $website['id'];
			$term_pages = isset( $termageddon_pages[ $site_id ] ) ? $termageddon_pages[ $site_id ] : array();
			?>
			<tr id="site-<?php echo intval( $site_id ); ?>" site-id="<?php echo intval( $site_id ); ?>">
				<td>
					<a href="admin.php?page=managesites&dashboard=<?php echo esc_html( $site_id ); ?>" target="_blank"><?php echo esc_html( $website['name'] ); ?></a>
				</td>
				<td>
				<a href="<?php echo esc_html( $website['url'] ); ?>" target="_blank"><?php echo esc_html( $website['url'] ); ?></a>
				</td>
				<?php
				foreach ( $terms as $term ) {
					$term_page_id = isset( $term_pages[ $term ] ) ? $term_pages[ $term ] : 0;
					?>
					<td id="termage-post-<?php echo intval( $term_page_id ); ?>">
					<?php
					if ( $term_page_id ) {
						?>
							<span class="ui tiny green label"><a href="admin.php?page=PageBulkEdit&termageddon=yes&post_id=<?php echo intval( $term_page_id ); ?>"><?php esc_html_e( 'Yes', 'termageddon-for-mainwp' ); ?></a></span> 
							<?php
					} else {
						?>
							<span class="ui tiny grey label"><a href="admin.php?page=PageBulkAdd&termageddon=yes&addnew=<?php echo esc_html( $term ); ?>&selected_sites=<?php echo intval( $site_id ); ?>"><?php esc_html_e( 'No', 'termageddon-for-mainwp' ); ?></a></span> 
							<?php
					}
					?>
					</td>
					<?php
				}
				?>
				<td class="right aligned">
					<div class="ui right pointing dropdown icon mini basic green button">
						<a href="javascript:void(0)"><i class="ellipsis horizontal icon"></i></a>
						<div class="menu">
							<a class="item" href="admin.php?page=managesites&dashboard=<?php echo $site_id; ?>"><?php _e( 'Overview', 'termageddon-for-mainwp' ); ?></a>
							<a class="item" href="admin.php?page=managesites&id=<?php echo $site_id; ?>"><?php _e( 'Edit', 'termageddon-for-mainwp' ); ?></a>
							<?php
							foreach ( $terms as $term ) {
								if ( isset( $term_pages[ $term ] ) ) {
									$term_page_id = $term_pages[ $term ];
									if ( empty( $term_page_id ) ) {
										continue;
									}
									$term_title = $this->get_title_of_term( $term )
									?>
									<a href="javascript:void(0)" termage-post-id=<?php echo intval( $term_page_id ); ?> class="item termage_posts_list_delete_post"><?php echo __( 'Delete', 'termageddon-for-mainwp' ) . ' ' . esc_html( $term_title ); ?></a>
									<?php
								}
							}
							?>
						</div>
					</div>
				</td>
			</tr>
		<?php endforeach; ?>
		<?php
	}

	public function get_title_of_term( $term ) {
		$terms = array(
			'privacy'    => __( 'Privacy Policy', 'termageddon-for-mainwp' ),
			'terms'      => __( 'Terms of Service', 'termageddon-for-mainwp' ),
			'disclaimer' => __( 'Disclaimer', 'termageddon-for-mainwp' ),
			'eula'       => __( 'EULA', 'termageddon-for-mainwp' ),
		);
		return isset( $terms[ $term ] ) ? $terms[ $term ] : '';
	}

}
