<?php
/*
 * Plugin Name: Termageddon for MainWP
 * Plugin URI: https://termageddon.com
 * Description: This extension is used for creating Privacy Policy, EULA, TOS and Disclaimer pages automatically on your websites.
 * Version: 1.0
 * Author: Termageddon
 * Author URI: https://termageddon.com
 * Documentation URI: https://termageddon.com
 * Icon URI: /wp-content/plugins/mainwp-termageddon-extension/images/termageddon-extension.png
 */

namespace Termageddon\MainWP;

if ( ! defined( 'TERMAGEDDON_FOR_MAINWP_URL' ) ) {
	define( 'TERMAGEDDON_FOR_MAINWP_URL', plugins_url( '', __FILE__ ) );
}

class Termageddon_For_MainWP_Activator {
	protected $mainwpMainActivated = false;
	protected $childEnabled        = false;
	protected $childKey            = false;
	protected $childFile;
	protected $plugin_handle    = 'termageddon-for-mainwp';
	protected $product_id       = 'Termageddon for MainWP';
	protected $software_version = '1.0';

	public function __construct() {

		$this->childFile = __FILE__;

		$this->includes();

		register_activation_hook( __FILE__, array( $this, 'activate' ) );
		register_deactivation_hook( __FILE__, array( $this, 'deactivate' ) );
		add_filter( 'mainwp_getextensions', array( &$this, 'get_this_extension' ) );

		$this->mainwpMainActivated = apply_filters( 'mainwp_activated_check', false );

		if ( false !== $this->mainwpMainActivated ) {
			$this->activate_this();
		} else {
			add_action( 'mainwp_activated', array( &$this, 'activate_this' ) );
		}
		add_action( 'admin_notices', array( &$this, 'mainwp_error_notice' ) );
	}

	public function includes() {
		require_once 'class/termageddon-for-mainwp-main.class.php';
		require_once 'class/termageddon-for-mainwp-page.class.php';
	}


	public function get_this_extension( $pArray ) {
		$pArray[] = array(
			'plugin'     => __FILE__,
			'api'        => $this->plugin_handle,
			'mainwp'     => false,
			'callback'   => array( &$this, 'render_main' ),
			'apiManager' => false,
		);

		return $pArray;
	}

	public function render_main() {
		do_action( 'mainwp_pageheader_extensions', __FILE__ );
		Termageddon_For_Mainwp_Page::get_instance()->render_pages_table();
		do_action( 'mainwp_pagefooter_extensions', __FILE__ );
	}


	public function activate_this() {
		$this->mainwpMainActivated = apply_filters( 'mainwp_activated_check', $this->mainwpMainActivated );
		$this->childEnabled        = apply_filters( 'mainwp_extension_enabled_check', __FILE__ );
		$this->childKey            = $this->childEnabled['key'];
		if ( function_exists( 'mainwp_current_user_can' ) && ! mainwp_current_user_can( 'extension', 'termageddon-for-mainwp' ) ) {
			return;
		}
		new Termageddon_For_Mainwp_Main();
	}

	public function get_enable_status() {
		return $this->childEnabled;
	}

	public function mainwp_error_notice() {
		global $current_screen;
		if ( 'plugins' == $current_screen->parent_base && false == $this->mainwpMainActivated ) {
			echo '<div class="error"><p>Termageddon for MainWP ' . __( 'requires <a href="https://mainwp.com/" target="_blank">MainWP Dashboard Plugin</a> to be activated in order to work. Please install and activate <a href="https://mainwp.com/" target="_blank">MainWP Dashboard Plugin</a> first.' ) . '</p></div>';
		}
	}

	public function get_child_key() {
		return $this->childKey;
	}

	public function get_child_file() {
		return $this->childFile;
	}

	public function activate() {
		$options = array(
			'product_id'       => $this->product_id,
			'software_version' => $this->software_version,
		);
		do_action( 'mainwp_activate_extention', $this->plugin_handle, $options );
	}

	public function deactivate() {
		do_action( 'mainwp_deactivate_extention', $this->plugin_handle );
	}
}

global $termageddonForMainWPActivator;
$termageddonForMainWPActivator = new Termageddon_For_MainWP_Activator();
